/*
 * disk-io  --  disk io monitor for mrtg (require Linux 2.6)
 * 
 * Copyright (C) 2006  Josef Kufner  <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 *
 *
 * Part of Linux kernel documentation (iostats.txt):
 *	
 *     Field  1 -- # of reads issued
 *         This is the total number of reads completed successfully.
 *     Field  2 -- # of reads merged, field 6 -- # of writes merged
 *         Reads and writes which are adjacent to each other may be merged for
 *         efficiency.  Thus two 4K reads may become one 8K read before it is
 *         ultimately handed to the disk, and so it will be counted (and queued)
 *         as only one I/O.  This field lets you know how often this was done.
 *     Field  3 -- # of sectors read
 *         This is the total number of sectors read successfully.
 *     Field  4 -- # of milliseconds spent reading
 *         This is the total number of milliseconds spent by all reads (as
 *         measured from __make_request() to end_that_request_last()).
 *     Field  5 -- # of writes completed
 *         This is the total number of writes completed successfully.
 *     Field  7 -- # of sectors written
 *         This is the total number of sectors written successfully.
 *     Field  8 -- # of milliseconds spent writing
 *         This is the total number of milliseconds spent by all writes (as
 *         measured from __make_request() to end_that_request_last()).
 *     Field  9 -- # of I/Os currently in progress
 *         The only field that should go to zero. Incremented as requests are
 *         given to appropriate request_queue_t and decremented as they finish.
 *     Field 10 -- # of milliseconds spent doing I/Os
 *         This field is increases so long as field 9 is nonzero.
 *     Field 11 -- weighted # of milliseconds spent doing I/Os
 *         This field is incremented at each I/O start, I/O completion, I/O
 *         merge, or read of these stats by the number of I/Os in progress
 *         (field 9) times the number of milliseconds spent doing I/O since the
 *         last update of this field.  This can provide an easy measure of both
 *         I/O completion time and the backlog that may be accumulating.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXLEN	256


int main (int argc, char *argv[]) {
	FILE *fs = NULL;
	char filename[MAXLEN];
	const char *template = "/sys/block/%s/stat";
	int i;
	unsigned long long stat[11], sector;


	/* check args */
	if (argc != 3) {
		fprintf(stderr, "Usage: %s disk sector_size\nExample: %s hda 512\n",
				argv[0], argv[0]);
		return 1;
	}
	sector = atoi(argv[2]);

	/* build filename */
	if (strlen(template) + strlen(argv[1]) > MAXLEN) {
		fprintf(stderr, "%s: Error: device name too long. Sorry.\n",
				argv[0]);
		return 2;
	}
	sprintf(filename, template, argv[1]);

	
	/* read stats
	 *
	 *     Field  0 -- # of reads issued
	 *     Field  1 -- # of reads merged
	 *     Field  2 -- # of sectors read
	 *     Field  3 -- # of milliseconds spent reading
	 *     Field  4 -- # of writes completed
	 *     Field  5 -- # of writes merged
	 *     Field  6 -- # of sectors written
	 *     Field  7 -- # of milliseconds spent writing
	 *     Field  8 -- # of I/Os currently in progress
	 *     Field  9 -- # of milliseconds spent doing I/Os
	 *     Field 10 -- weighted # of milliseconds spent doing I/Os
	 */
	fs = fopen(filename, "r");
	if (!fs) {
		perror(filename);
		return 3;
	}
	for (i = 0; i <= 10 && fscanf(fs, "%llu", &(stat[i])); i++)
		;

	/* print stat */
	printf("%llu\n%llu\n", stat[2] * sector, stat[6] * sector);

	return 0;
}


